
import sys
import logging
import logging.config
import io
import yaml


from .logxintrospect import get_nicest_module_name


YAML_CONFIG_PATH = 'loggers.yaml'

DEFAULT_FORMAT = \
    '[%(asctime)s] %(levelname)s [%(name)s.%(funcName)s:%(lineno)d] %(message)s'


def load_yaml_config(yaml_config_path=YAML_CONFIG_PATH):
    with io.open(yaml_config_path) as f:
        yaml_config_text = f.read()

    yaml_config = yaml.load(yaml_config_text)
    logging.config.dictConfig(yaml_config)


class StdHandler(logging.Handler):
    def emit(self, record):
        msg = self.format(record)

        if record.levelno >= logging.WARNING:
            stream = sys.stderr
        else:
            stream = sys.stdout

        stream.write(msg)
        stream.write('\n')
        self.flush()


def set_basic_config():
    logging.basicConfig(level=logging.DEBUG)


def set_root_level(level='DEBUG'):
    get_logger('').setLevel(getattr(logging, level))


def get_logger(name):
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    return logger


def print_diagram():
    import logging_tree
    logging_tree.printout()


class Log():
    def __init__(self):
        self.handlers = {}

    def set_level(self, level, name=None):
        handler = self.get_handler(name)
        handler.setLevel(getattr(logging, level.upper()))

    def add_std_handler(self, logger_name, level='DEBUG', top_level=True):
        if top_level:
            logger_name = logger_name.split('.')[0]

        level = level.upper()
        logging_level = getattr(logging, level)

        logger = logging.getLogger(logger_name)

        logger.setLevel(logging_level)
        h = StdHandler()
        h.setLevel(logging_level)
        logger.addHandler(h)

        return h

    def current_logger_name(self):
        name = get_nicest_module_name()
        return name

    def set_default_format(self, name=None):
        self.set_format(DEFAULT_FORMAT)

    def get_logger(self, name=None):
        name = name or self.current_logger_name()
        return logging.getLogger(name)

    def get_handler(self, name=None, top_level=True):
        name = name or self.current_logger_name()

        if top_level:
            name = name.split('.')[0]

        logger = self.get_logger(name)

        for h in logger.handlers:
            if isinstance(h, (StdHandler, logging.NullHandler)):
                return h

        raise KeyError('no logx-related handler found on this logger')

    def set_format(self, formatstring, logger_name=None, top_level=True):
        logger_name = logger_name or self.current_logger_name()

        if top_level:
            logger_name = logger_name.split('.')[0]

        try:
            handler = self.get_handler(logger_name)
        except KeyError:
            handler = None

        if not handler:
            handler = self.add_std_handler(logger_name)

        formatter = logging.Formatter(formatstring)
        handler.setFormatter(formatter)

    def set_null_handler(self, name=None, top_level=True):
        name = name or self.current_logger_name()

        if top_level:
            name = name.split('.')[0]

        logger = self.get_logger(name=name)

        logger.handlers = [
            _ for _ in logger.handlers
            if not isinstance(_, StdHandler)
        ]

        logger.addHandler(logging.NullHandler())
        print_diagram()

    def clear_null_handler(self, name=None, top_level=True):
        name = name or self.current_logger_name()

        if top_level:
            name = name.split('.')[0]

        logger = self.get_logger(name=name)

        logger.handlers = [
            _ for _ in logger.handlers
            if not isinstance(_, logging.NullHandler)
        ]

    def __getattr__(self, method, *args, **kwargs):
        if method.startswith('_'):
            raise AttributeError
        logger_name = self.current_logger_name()

        try:
            self.get_handler(logger_name)
        except KeyError:
            self.add_std_handler(logger_name)

        _logger = logging.getLogger(logger_name)
        return getattr(_logger, method)




import inspect


def get_calling_module(name=None, top_level_only=True):
    def top_module(mname):

        if top_level_only:
            return mname.split('.')[0]
        else:
            return mname
    s = inspect.stack()

    def modules_iter():
        for i in range(len(s)):
            m = inspect.getmodule(s[i][0])
            # func_name = s[i][3]
            yield m

    x = next((
        m for m in modules_iter()
        if
        m and
        top_module(m.__name__) != top_module(__name__)),
        None
    )

    return x


def get_nicest_module_name(m=None):
    if not m:
        m = get_calling_module()

    if m:
        _name = m.__name__
    else:
        return '__main__'

    def is_main(name):
        return name == '__main__'

    if is_main(_name):
        if m.__spec__:
            return m.__spec__.name
        else:
            return inspect.getmodulename(inspect.getfile(m))
    else:
        return _name




from logx import log, print_diagram


def load_config(path):
    import io
    import os
    import yaml
    import logging

    if path and os.path.exists(path):
        with io.open(path) as f:
            conf = f.read()
        logging.config.dictConfig(yaml.load(conf))


def m(caplog):
    return caplog.records[-1].message


def test_plain_output(caplog, capsys):
    logger_name = log.current_logger_name()

    assert logger_name == 'tests.test_logx'
    FIRST_MESSAGE = 'first message'

    log.info(FIRST_MESSAGE)
    assert m(caplog) == FIRST_MESSAGE

    out, err = capsys.readouterr()

    assert out == FIRST_MESSAGE + '\n'
    assert not err


def test_formatted_output(caplog, capsys):
    log.set_default_format()

    log.debug('debug')

    assert m(caplog) == 'debug'

    out, err = capsys.readouterr()
    assert out.endswith('debug\n')

    assert 'DEBUG [tests.test_logx.test_formatted_output:' in out
    assert not err


def test_level_change_output(caplog, capsys):
    log.set_default_format()

    log.set_level('warn')

    log.warn('warn')
    log.debug('debug')

    out, err = capsys.readouterr()

    assert 'warn' in err
    assert 'debug' not in err and 'debug' not in out


def test_null_handler(caplog, capsys):
    log.set_null_handler()

    log.warn('warn')
    log.debug('debug')

    out, err = capsys.readouterr()

    assert 'warn' not in err and 'warn' not in out
    assert 'debug' not in err and 'debug' not in out

    log.clear_null_handler()
    print_diagram()
    log.warn('warn')
    log.debug('debug')

    out, err = capsys.readouterr()

    assert 'warn' in err and 'warn' not in out
    assert 'debug' not in err and 'debug' in out



#!/usr/bin/env python

import io

from setuptools import setup, find_packages

readme = io.open('README.md').read()

setup(
    name='logx',
    version='0.1.1519641991',
    url='https://github.com/djrobstep/logx',
    description='best practice python logging with zero config',
    long_description=readme,
    author='Robert Lechte',
    author_email='robertlechte+djrobstep@gmail.com',
    install_requires=[
        'logging_tree',
        'pyyaml'
    ],
    zip_safe=False,
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
    ]
)
